"""add last login

Revision ID: 4099fa344171
Revises: 139ec7ed17b7
Create Date: 2013-10-24 10:13:45.104902

"""

import sqlalchemy as sa
from alembic import op


# revision identifiers, used by Alembic.
revision = "4099fa344171"
down_revision = "139ec7ed17b7"


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column("persons", sa.Column("last_login", sa.DateTime(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("persons", "last_login")
    ### end Alembic commands ###
